/*<<dependency*/
define("biexcellence_openbi_sap_lumira_viz_exportppt-src/js/render", ["openbiGetHtml"], function () {
    /*dependency>>*/
    /*
     * This function is a drawing function; you should put all your drawing logic in it.
     * it's called in moduleFunc.prototype.render
     * @param {Object} data - proceessed dataset, check dataMapping.js
     * @param {Object} container - the target d3.selection element of plot area
     * @example
     *   container size:     this.width() or this.height()
     *   chart properties:   this.properties()
     *   dimensions info:    data.meta.dimensions()
     *   measures info:      data.meta.measures()
     */
    var render = function (data, container) {
        var width = this.width(),
            height = this.height(),
            colorPalette = this.colorPalette(),
            properties = this.properties(),
            dispatch = this.dispatch();


        // initialize
        var lmodule = properties.exportFormat;
        if (lmodule == "PowerPoint") {
            lmodule = "ppt";
        }
        if (lmodule == "Word") {
            lmodule = "doc";
        }
        if (lmodule == "Excel") {
            lmodule = "xls";
        }
        if (lmodule == "PDF") {
            lmodule = "pdf";
        }
        
        var filePath = '../sap/bi/bundles/biexcellence/openbi/sap/lumira/viz/exportppt';


        //prepare canvas with width and height of container
        container.node().innerHTML = "";
        container.selectAll("div").remove();
        var vis = container.append("div").attr("width", width).attr("height", height).attr("id", "openbiExport" + lmodule).attr("class", "openbihideonprint");


        // render visualization
        // export icon
        var ppt_button = vis.append("div");
        ppt_button.attr("class", "openbi" + lmodule.toUpperCase() + "Export" + properties.renderingMode).attr("id", "openbiExport" + lmodule);

        if (lmodule == "ppt") {
            ppt_button.on("click", function (d) {
                doExport("PPTX");
            });
        }
        if (lmodule == "xls") {
            ppt_button.on("click", function (d) {
                doExport("XLSX");
            });
        }
        if (lmodule == "doc") {
            ppt_button.on("click", function (d) {
                doExport("DOCX");
            });
        }
        if (lmodule == "pdf") {
            ppt_button.on("click", function (d) {
                doExport("PDF");
            });
        }

        // render loading indicator
        //var export_loading = vis.append("div");
        //export_loading.attr("class", "openbiLoading").attr("id", "bie_openbi_export_loading_" + lmodule).attr("name", "bie_openbi_export_loading");

        // render form
        var form = vis.append("form").attr("id", "bie_openbi_export_form_" + lmodule).attr("name", "bie_openbi_export_form_" + lmodule).attr("method", "post").attr("accept-charset", "utf-8").attr("action", properties.serverUrl + "/lumira/lumiraexport2.html");
        form.append("input").attr("id", "bie_openbi_export_format").attr("name", "bie_openbi_export_format").attr("hidden", true);
        form.append("input").attr("id", "bie_openbi_export_" + lmodule + "_template").attr("name", "bie_openbi_export_" + lmodule + "_template").attr("hidden", true).attr("value", properties.template);
        form.append("input").attr("id", "bie_openbi_export_parsecss").attr("name", "bie_openbi_export_parsecss").attr("hidden", true).attr("value", properties.parseCss);

        if ((lmodule == "ppt") || (lmodule == "pdf")) {
            var lvalue = "";
            if (properties.separateSlides) {
                lvalue = 'X';
            }
            form.append("input").attr("id", "bie_openbi_export_ppt_separate").attr("name", "bie_openbi_export_ppt_separate").attr("hidden", true).attr("value",lvalue);

        }

    };

    return render;

});



// export function
function doExport(iformat) {

    var lmodule = iformat.substring(0, 3).toLowerCase();
    var form = document.getElementById("bie_openbi_export_form_" + lmodule);
    var lremove = [];

    // loading indicator
    //var lloadingtext = document.createTextNode("Collecting Resources...");
    //document.getElementById("bie_openbi_export_loading_" + lmodule).appendChild(lloadingtext);
    document.getElementById("openbiExport" + lmodule).style.webkitFilter = "grayscale(100%)";
    document.getElementById("openbiExport" + lmodule).style.filter = "grayscale(100%)";
    document.getElementById("openbiExport" + lmodule).style.color = "gray";

    var lparsecss = false;

    for ( var i = 0; i < form.childNodes.length; i++) {
        var lnode = form.childNodes[i];
        switch ( lnode.name ) {
            // format
            case "bie_openbi_export_parsecss":
                if (lnode.value == "true") {
                    lparsecss = true;
                }
                break;

            case "bie_openbi_export_format":
                lnode.value = iformat;
                break;

            case "bie_openbi_export_content":
                lremove.push(lnode);
                break;
        }
    }

    if ( lremove.length > 0 ) {
        for (var i = 0; i < lremove.length; i++) {
            form.removeChild(lremove[i]);
            // document.getElementById("openbiExport" + lmodule).remove(document.getElementById("openbiExport" + lmodule).lastChild);
        }
    }

    // content HTML 
    var lrequire = "./zen/mimes/sdk_include/biexcellence.openbi.sap.lumira.viz.exportppt/bundles/biexcellence/openbi/sap/lumira/viz/exportppt/openbiGetHtml.js";
    //var lrequire = "./zen/mimes/sdk_include/biexcellence.openbi.sap.lumira.viz.export" + lmodule + "/bundles/biexcellence/openbi/sap/lumira/viz/export" + lmodule + "/openbiGetHtml.js";
    require([lrequire], function (processHtml) {
        processHtml(lparsecss, false, function (content_html){

            var form = document.getElementById("bie_openbi_export_form_" + lmodule);
            
            content_html = content_html.replace(/openbidohide openbionoverflow/g, "openbionoverflow");
            var lparts = Math.ceil( content_html.length / 500000 );
            
            for (var i = 0; i < lparts; i++ ) {
                // add form element
                var linput = document.createElement("input"); //input element, text
                linput.setAttribute('name', "bie_openbi_export_content");
                linput.setAttribute('id', "bie_openbi_export_content_" + i.toString());
                linput.setAttribute('type', "hidden");
                linput.setAttribute('value', "");

                linput.value = content_html.substring(i * 500000, ( i + 1 ) * 500000 );
                form.appendChild(linput);

            }

            // create explicit iframe
            var liframe = document.createElement("iframe");
            liframe.setAttribute('id', "bie_openbi_export_iframe_" + lmodule);
            liframe.setAttribute('name', "bie_openbi_export_iframe_" + lmodule);
            liframe.setAttribute('style', "display:none;");
            document.getElementById("openbiExport" + lmodule).appendChild(liframe);

            // do export
            //form.action = document.getElementById("openbiServerUrl").value + "/lumira/lumiraexport.html";
            if (form.action.indexOf("https") > -1) {
                form.target = "_blank";
            } else {
                form.target = "bie_openbi_export_iframe_" + lmodule;
            }
            form.submit();

            // loading indicator - replace this static timeout with AJAX function (task 2590)
            setTimeout(function () {
                document.getElementById("openbiExport" + lmodule).style.webkitFilter = ""
                document.getElementById("openbiExport" + lmodule).style.filter = ""
                document.getElementById("openbiExport" + lmodule).style.color = "";
            }, 3000);

        });
    });

}


function onErrorMessage(imessage) {
        alert(imessage);
 }

