﻿(function (define) {
define(function () {

    // https://github.com/niklasvh/html2canvas/issues/503
    // https://github.com/niklasvh/html2canvas/blob/master/src/clone.js#L10-L44
    function cloneCanvasContents(canvas, clonedCanvas) {
        try {
            if (clonedCanvas) {
                clonedCanvas.width = canvas.width;
                clonedCanvas.height = canvas.height;
                clonedCanvas.getContext("2d").putImageData(canvas.getContext("2d").getImageData(0, 0, canvas.width, canvas.height), 0, 0);
            }
        } catch (e) {

        }
    }
    function cloneNode(node, javascriptEnabled) {
        var clone = node.nodeType === 3 ? document.createTextNode(node.nodeValue) : node.cloneNode(false);

        var child = node.firstChild;
        while (child) {
            if (javascriptEnabled === true || child.nodeType !== 1 || child.nodeName !== 'SCRIPT') {
                clone.appendChild(cloneNode(child, javascriptEnabled));
            }
            child = child.nextSibling;
        }

        if (node.nodeType === 1) {
            var data;

            if ((data = node.scrollTop)) {
                clone.setAttribute("data-scroll-top", data);
            }
            if ((data = node.scrollLeft)) {
                clone.setAttribute("data-scroll-left", data);
            }

            if (node.nodeName === "CANVAS") {
                cloneCanvasContents(node, clone);
            } else if (node.nodeName === "INPUT") {
                clone.setAttribute("value", node.value);
                clone.removeAttribute("checked"); // handle type checkbox and radio
                if (node.checked) {
                    clone.setAttribute("checked", "checked");
                }
            } else if (node.nodeName === "TEXTAREA") {
                clone.textContent = node.value;
            } else if (node.nodeName === "OPTION") {
                clone.removeAttribute("selected");
                if (node.selected) {
                    clone.setAttribute("selected", "selected");
                }
            }
        }

        return clone;
    }

    function getHtml(parseCssImageEle, parseCssThirdEle, cb) {
        var dom = document.documentElement,

        //var dom = document.documentElement.getElementsByClassName("sapLumiraStoryLayoutUnifiedComponentPageContainer")[0],
        newDom = cloneNode(dom, false),

            $styles = $("style,link", dom),
            $newStyles = $("style,link", newDom),
            $canvas = $("canvas", dom),
            $newCanvas = $("canvas", newDom),
            $imgs = $("img", dom),
            $newImgs = $("img", newDom),

            async = createAsync(function () {
                cb(newDom.outerHTML);
            });

        var parseImages = parseCssImageEle;
        var parseThird = parseCssThirdEle;

        // get inline background images
        if (parseThird) {

            var $inlineStyles = $("[style*='background-image']", dom),
                $newInlineStyles = $("[style*='background-image']", newDom);

            $inlineStyles.each(function (i, el) {
                if (!el.style.backgroundImage) return;

                if ((el.style.backgroundImage.indexOf("url") == 0) && (el.style.backgroundImage.indexOf("url(data:") != 0)) {

                    var newInlineStyle = $newInlineStyles.get(i);

                    if (newInlineStyle.style.backgroundImage) {

                        var baseUrl = newInlineStyle.href || document.location.href,
                            urlParts = baseUrl.split("/"),
                            urlPartsLength = urlParts.length,
                            cssUrl = /url\(["']?(.*?)["']?\)/ig,
                            urls = [];

                        if (urlParts[urlPartsLength - 1].indexOf(".") != -1) {
                            urlParts[urlPartsLength - 1] = "";
                            baseUrl = urlParts.join("/");
                        }

                        var css = newInlineStyle.style.backgroundImage.replace(cssUrl, function (match, found, index, all) {
                            var url = found;

                            if (url.indexOf("data:") === 0) return match;

                            if (!/^http/i.test(url)) {
                                if (baseUrl.lastIndexOf("/") == baseUrl.length - 1 && url.indexOf("/") == 0) {
                                    url = baseUrl + url.substring(1);
                                } else {
                                    url = baseUrl + url;
                                }
                            }

                            if (urls.indexOf(url) === -1) {
                                urls.push(url);
                            }

                            return match.replace(found, url);
                        });

                        $.each(urls, function (i, url) {
                            var img = new Image();

                            img.onload = img.onerror = async(function () {
                                if (img.complete) {
                                    getImageDataUrl(img, async(function (dataUrl) {
                                        if (dataUrl.indexOf("data:") == 0) {
                                            newInlineStyle.style.backgroundImage = css = css.split(url).join(dataUrl);
                                        } else {
                                            newInlineStyle.style.backgroundImage = "";
                                        }
                                    }));
                                }
                            });

                            img.src = url;
                        });
                    }

                    newInlineStyle.style.backgroundImage = css;
                }
            });
        }

        // get css styles & images
        $styles.each(function (i, style) {
 
            var newStyle = $newStyles.get(i),
                css = style.innerHTML || getCssText(style.sheet || style.styleSheet); // handle <style> and <link>

            if (css) {
                if (style.tagName.toUpperCase() != "STYLE") {
                    var temp = document.createElement("style"),
                        newParent = newStyle.parentNode;

                    newParent.insertBefore(temp, newStyle);
                    newParent.removeChild(newStyle);

                    newStyle = temp;
                }

                if (parseImages) {
                    var baseUrl = style.href || document.location.href,
                        urlParts = baseUrl.split("/"),
                        urlPartsLength = urlParts.length,
                        cssUrl = /url\(["']?(.*?)["']?\)/ig,
                        urls = [];

                    if (urlParts[urlPartsLength - 1].indexOf(".") != -1) {
                        urlParts[urlPartsLength - 1] = "";
                        baseUrl = urlParts.join("/");
                    }

                    css = css.replace(cssUrl, function (match, found, index, all) {
                        var url = found;

                        if (url.indexOf("data:") === 0) return match;

                        if (!/^http/i.test(url)) {
                            if (baseUrl.lastIndexOf("/") == baseUrl.length - 1 && url.indexOf("/") == 0) {
                                url = baseUrl + url.substring(1);
                            } else {
                                url = baseUrl + url;
                            }
                        }

                        if (urls.indexOf(url) === -1) {
                            urls.push(url);
                        }

                        return match.replace(found, url);
                    });

                    $.each(urls, function (i, url) {
                        var img = new Image();

                        img.onload = img.onerror = async(function () {
                            if (img.complete) {
                                getImageDataUrl(img, async(function (dataUrl) {
                                    newStyle.innerHTML = css = css.split(url).join(dataUrl);
                                }));
                            }
                        });

                        img.src = url;
                    });
                }

                newStyle.innerHTML = css;
            }
        });

        // get canvas elements
        $canvas.each(function (i, canvas) {
            var newCanvas = $newCanvas.get(i),
                newParent = newCanvas.parentNode,
                img = document.createElement("img");

            $.each(canvas.attributes, function (j, attribute) {
                img.setAttribute(attribute.name, attribute.value);
            });
            img.setAttribute("src", canvas.toDataURL("image/png"));

            newParent.insertBefore(img, newCanvas);
            newParent.removeChild(newCanvas);
        });

        // get images
        $imgs.each(function (i, img) {
            if (!img.src) return;
            // if parse CSS is set get all images, also of 3rd party components!
            if (!parseThird && (img.src.indexOf(location.origin || (location.protocol + "//" + location.host)) != 0)) return;

            var newImg = $newImgs.get(i);

            getImageDataUrl(img, async(function (dataUrl) {
                if ((dataUrl.indexOf("data:") === 0) || (img.src.indexOf(location.origin || (location.protocol + "//" + location.host)) === 0)) {
                    newImg.setAttribute("src", dataUrl);
                } else {
                    newImg.setAttribute("src", "");
                }
            }));
        });

        async(function noop() { })();
    }

    function getCssText(sheet) {
        var rules, cssText = "";
        try {
            rules = sheet && (sheet.cssRules || sheet.rules);
        } catch (e) { }

        if (rules) {
            $.each(rules, function (i, rule) {
                if (rule.styleSheet) { // handle @import
                    cssText += getCssText(rule.styleSheet)
                } else {
                    var value;
                    if (rule.type == 5) { // CSSRule.FONT_FACE_RULE
                        try {
                            value = rule.style.getPropertyValue("src");
                            rule.style.removeProperty("src");
                        } catch (e) {
                            try {
                                value = rule.style.src;
                                rule.style.src = "";
                            } catch (e) { }
                        }
                    }

                    try {
                        cssText += rule.cssText;
                    } catch (e) {
                        try {
                            cssText += rule.style.cssText;
                        } catch (e) { }
                    }

                    if (rule.type == 5) { // CSSRule.FONT_FACE_RULE
                        try {
                            rule.style.setProperty("src", value);
                        } catch (e) {
                            try {
                                rule.style.src = value;
                            } catch (e) { }
                        }
                    }
                }
            });
        }

        return cssText;
    }

    function getImageDataUrl(img, cb) {
        try {
            var canvas = document.createElement("canvas"),
                ctx = canvas.getContext("2d"),
                width = canvas.width = img.width,
                height = canvas.height = img.height;

            getImageDimensions(img, function (actualWidth, actualHeight) {
                try {
                    ctx.drawImage(img, 0, 0, actualWidth, actualHeight, 0, 0, width, height);

                    cb(canvas.toDataURL("image/png"));
                } catch (e) {
                    cb(img.src);
                }
            });
        } catch (e) {
            cb(img.src);
        }
    }
    function getImageDimensions(img, cb) {
        if ("naturalWidth" in img && "naturalHeight" in img) {
            cb(img.naturalWidth, img.naturalHeight);
        } else {
            var temp = new Image();
            temp.onload = function () {
                cb(temp.width, temp.height);
            };
            temp.onerror = function () {
                cb(img.width, img.height);
            };
            temp.src = img.src;
        }
    }
    function createAsync(cb) {
        var counter = 0, timeout;
        return function (fn) {
            counter++;
            return function () {
                counter--;
                fn.apply(null, arguments);
                if (counter == 0) {
                    clearTimeout(timeout);
                    timeout = setTimeout(function () {
                        cb();
                        cb = function () { };
                    }, 50);
                }
            };
        };
    }

	return getHtml;
});
})(
    // commonjs & node.js
    /*typeof module === "object" && */typeof /*module.*/exports === "object" ?
        function (id, deps, factory) {
        	module.exports = factory();
        } :
    // amd
    typeof define === "function" && define.amd ?
	define :
    // <script>
    function (id, deps, factory) {
    	window.getHtml = factory();
    }
);

function adjustPrintParameters(name, value, inumber) {
    if (document.getElementsByName("bie_openbi_export_form")[inumber].elements[name] != null) {
        document.getElementsByName("bie_openbi_export_form")[inumber].elements[name].value = value;
    } else {
        var input = document.createElement("input");
        input.type = "hidden";
        input.name = name;
        input.value = value;
        document.getElementsByName("bie_openbi_export_form")[inumber].appendChild(input);
    }
}
